package sun.java2d.loops;

import java.util.Vector;
import java.util.List;
import java.awt.geom.PathIterator;
import java.util.Arrays;
import java.awt.geom.QuadCurve2D;
import java.awt.geom.Path2D;

public class ProcessPath
{
    public static final int PH_MODE_DRAW_CLIP = 0;
    public static final int PH_MODE_FILL_CLIP = 1;
    public static ProcessPath.EndSubPathHandler noopEndSubPathHandler;
    private static final float UPPER_BND = 8.5070587E37f;
    private static final float LOWER_BND = -8.5070587E37f;
    private static final int FWD_PREC = 7;
    private static final int MDP_PREC = 10;
    private static final int MDP_MULT = 1024;
    private static final int MDP_HALF_MULT = 512;
    private static final int UPPER_OUT_BND = 1048576;
    private static final int LOWER_OUT_BND = -1048576;
    private static final float CALC_UBND = 1048576.0f;
    private static final float CALC_LBND = -1048576.0f;
    public static final int EPSFX = 1;
    public static final float EPSF = 9.765625E-4f;
    private static final int MDP_W_MASK = -1024;
    private static final int MDP_F_MASK = 1023;
    private static final int MAX_CUB_SIZE = 256;
    private static final int MAX_QUAD_SIZE = 1024;
    private static final int DF_CUB_STEPS = 3;
    private static final int DF_QUAD_STEPS = 2;
    private static final int DF_CUB_SHIFT = 6;
    private static final int DF_QUAD_SHIFT = 1;
    private static final int DF_CUB_COUNT = 8;
    private static final int DF_QUAD_COUNT = 4;
    private static final int DF_CUB_DEC_BND = 262144;
    private static final int DF_CUB_INC_BND = 32768;
    private static final int DF_QUAD_DEC_BND = 8192;
    private static final int DF_QUAD_INC_BND = 1024;
    private static final int CUB_A_SHIFT = 7;
    private static final int CUB_B_SHIFT = 11;
    private static final int CUB_C_SHIFT = 13;
    private static final int CUB_A_MDP_MULT = 128;
    private static final int CUB_B_MDP_MULT = 2048;
    private static final int CUB_C_MDP_MULT = 8192;
    private static final int QUAD_A_SHIFT = 7;
    private static final int QUAD_B_SHIFT = 9;
    private static final int QUAD_A_MDP_MULT = 128;
    private static final int QUAD_B_MDP_MULT = 512;
    private static final int CRES_MIN_CLIPPED = 0;
    private static final int CRES_MAX_CLIPPED = 1;
    private static final int CRES_NOT_CLIPPED = 3;
    private static final int CRES_INVISIBLE = 4;
    private static final int DF_MAX_POINT = 256;
    
    public ProcessPath() {
    }
    
    public static boolean fillPath(ProcessPath.DrawHandler drawHandler, Path2D.Float float1, int n, int n2) {
        ProcessPath.FillProcessHandler fillProcessHandler = new FillProcessHandler(drawHandler);
        if (!doProcessPath(fillProcessHandler, float1, (float)n, (float)n2)) {
            return false;
        }
        FillPolygon(fillProcessHandler, float1.getWindingRule());
        return true;
    }
    
    public static boolean drawPath(ProcessPath.DrawHandler drawHandler, ProcessPath.EndSubPathHandler endSubPathHandler, Path2D.Float float1, int n, int n2) {
        return doProcessPath(new ProcessPath.DrawProcessHandler(drawHandler, endSubPathHandler), float1, (float)n, (float)n2);
    }
    
    public static boolean drawPath(ProcessPath.DrawHandler drawHandler, Path2D.Float float1, int n, int n2) {
        return doProcessPath(new ProcessPath.DrawProcessHandler(drawHandler, ProcessPath.noopEndSubPathHandler), float1, (float)n, (float)n2);
    }
    
    private static float CLIP(float n, float n2, float n3, float n4, double n5) {
        return (float)((double)n2 + (n5 - (double)n) * (double)(n4 - n2) / (double)(n3 - n));
    }
    
    private static int CLIP(int n, int n2, int n3, int n4, double n5) {
        return (int)((double)n2 + (n5 - (double)n) * (double)(n4 - n2) / (double)(n3 - n));
    }
    
    private static boolean IS_CLIPPED(int n) {
        return n == 0 || n == 1;
    }
    
    private static int TESTANDCLIP(float n, float n2, float[] array, int n3, int n4, int n5, int n6) {
        int n7 = 3;
        if (array[n3] < n || array[n3] > n2) {
            double n8;
            if (array[n3] < n) {
                if (array[n5] < n) {
                    return 4;
                }
                n7 = 0;
                n8 = (double)n;
            }
            else {
                if (array[n5] > n2) {
                    return 4;
                }
                n7 = 1;
                n8 = (double)n2;
            }
            array[n4] = CLIP(array[n3], array[n4], array[n5], array[n6], n8);
            array[n3] = (float)n8;
        }
        return n7;
    }
    
    private static int TESTANDCLIP(int n, int n2, int[] array, int n3, int n4, int n5, int n6) {
        int n7 = 3;
        if (array[n3] < n || array[n3] > n2) {
            double n8;
            if (array[n3] < n) {
                if (array[n5] < n) {
                    return 4;
                }
                n7 = 0;
                n8 = (double)n;
            }
            else {
                if (array[n5] > n2) {
                    return 4;
                }
                n7 = 1;
                n8 = (double)n2;
            }
            array[n4] = CLIP(array[n3], array[n4], array[n5], array[n6], n8);
            array[n3] = (int)n8;
        }
        return n7;
    }
    
    private static int CLIPCLAMP(float n, float n2, float[] array, int n3, int n4, int n5, int n6, int n7, int n8) {
        array[n7] = array[n3];
        array[n8] = array[n4];
        int testandclip = TESTANDCLIP(n, n2, array, n3, n4, n5, n6);
        if (testandclip == 0) {
            array[n7] = array[n3];
        }
        else if (testandclip == 1) {
            array[n7] = array[n3];
            testandclip = 1;
        }
        else if (testandclip == 4) {
            if (array[n3] > n2) {
                testandclip = 4;
            }
            else {
                array[n5] = (array[n3] = n);
                testandclip = 3;
            }
        }
        return testandclip;
    }
    
    private static int CLIPCLAMP(int n, int n2, int[] array, int n3, int n4, int n5, int n6, int n7, int n8) {
        array[n7] = array[n3];
        array[n8] = array[n4];
        int testandclip = TESTANDCLIP(n, n2, array, n3, n4, n5, n6);
        if (testandclip == 0) {
            array[n7] = array[n3];
        }
        else if (testandclip == 1) {
            array[n7] = array[n3];
            testandclip = 1;
        }
        else if (testandclip == 4) {
            if (array[n3] > n2) {
                testandclip = 4;
            }
            else {
                array[n5] = (array[n3] = n);
                testandclip = 3;
            }
        }
        return testandclip;
    }
    
    private static void DrawMonotonicQuad(ProcessPath.ProcessHandler processHandler, float[] array, boolean b, int[] array2) {
        int n = (int)(array[0] * 1024.0f);
        int n2 = (int)(array[1] * 1024.0f);
        int n3 = (int)(array[4] * 1024.0f);
        int n4 = (int)(array[5] * 1024.0f);
        int n5 = (n & 0x3FF) << 1;
        int n6 = (n2 & 0x3FF) << 1;
        int n7 = 4;
        int n8 = 1;
        int n9 = (int)((array[0] - 2.0f * array[2] + array[4]) * 128.0f);
        int n10 = (int)((array[1] - 2.0f * array[3] + array[5]) * 128.0f);
        int n11 = (int)((-2.0f * array[0] + 2.0f * array[2]) * 512.0f);
        int n12 = (int)((-2.0f * array[1] + 2.0f * array[3]) * 512.0f);
        int a = 2 * n9;
        int a2 = 2 * n10;
        int n13 = n9 + n11;
        int n14 = n10 + n12;
        int i = Math.max(Math.abs(a), Math.abs(a2));
        int n15 = n3 - n;
        int n16 = n4 - n2;
        int n17 = n & 0xFFFFFC00;
        int n18 = n2 & 0xFFFFFC00;
        while (i > 8192) {
            n13 = (n13 << 1) - n9;
            n14 = (n14 << 1) - n10;
            n7 <<= 1;
            i >>= 2;
            n5 <<= 2;
            n6 <<= 2;
            n8 += 2;
        }
        while (n7-- > 1) {
            n5 += n13;
            n6 += n14;
            n13 += a;
            n14 += a2;
            n = n17 + (n5 >> n8);
            n2 = n18 + (n6 >> n8);
            if ((n3 - n ^ n15) < 0) {
                n = n3;
            }
            if ((n4 - n2 ^ n16) < 0) {
                n2 = n4;
            }
            processHandler.processFixedLine(n, n2, n, n2, array2, b, false);
        }
        processHandler.processFixedLine(n, n2, n3, n4, array2, b, false);
    }
    
    private static void ProcessMonotonicQuad(ProcessPath.ProcessHandler processHandler, float[] array, int[] array2) {
        float[] array3 = new float[6];
        float n2;
        float n = n2 = array[0];
        float n4;
        float n3 = n4 = array[1];
        for (int i = 2; i < 6; i += 2) {
            n2 = ((n2 > array[i]) ? array[i] : n2);
            n = ((n < array[i]) ? array[i] : n);
            n4 = ((n4 > array[i + 1]) ? array[i + 1] : n4);
            n3 = ((n3 < array[i + 1]) ? array[i + 1] : n3);
        }
        if (processHandler.clipMode == 0) {
            if (processHandler.dhnd.xMaxf < n2 || processHandler.dhnd.xMinf > n || processHandler.dhnd.yMaxf < n4 || processHandler.dhnd.yMinf > n3) {
                return;
            }
        }
        else {
            if (processHandler.dhnd.yMaxf < n4 || processHandler.dhnd.yMinf > n3 || processHandler.dhnd.xMaxf < n2) {
                return;
            }
            if (processHandler.dhnd.xMinf > n) {
                int n5 = 0;
                int n6 = 2;
                int n7 = 4;
                float xMinf = processHandler.dhnd.xMinf;
                array[n7] = xMinf;
                array[n5] = (array[n6] = xMinf);
            }
        }
        if (n - n2 > 1024.0f || n3 - n4 > 1024.0f) {
            array3[4] = array[4];
            array3[5] = array[5];
            array3[2] = (array[2] + array[4]) / 2.0f;
            array3[3] = (array[3] + array[5]) / 2.0f;
            array[2] = (array[0] + array[2]) / 2.0f;
            array[3] = (array[1] + array[3]) / 2.0f;
            array[4] = (array3[0] = (array[2] + array3[2]) / 2.0f);
            array[5] = (array3[1] = (array[3] + array3[3]) / 2.0f);
            ProcessMonotonicQuad(processHandler, array, array2);
            ProcessMonotonicQuad(processHandler, array3, array2);
        }
        else {
            DrawMonotonicQuad(processHandler, array, processHandler.dhnd.xMinf >= n2 || processHandler.dhnd.xMaxf <= n || processHandler.dhnd.yMinf >= n4 || processHandler.dhnd.yMaxf <= n3, array2);
        }
    }
    
    private static void ProcessQuad(ProcessPath.ProcessHandler processHandler, float[] array, int[] array2) {
        double[] array3 = new double[2];
        int n = 0;
        if ((array[0] > array[2] || array[2] > array[4]) && (array[0] < array[2] || array[2] < array[4])) {
            double n2 = (double)(array[0] - 2.0f * array[2] + array[4]);
            if (n2 != 0.0) {
                double n3 = (double)(array[0] - array[2]) / n2;
                if (n3 < 1.0 && n3 > 0.0) {
                    array3[n++] = n3;
                }
            }
        }
        if ((array[1] > array[3] || array[3] > array[5]) && (array[1] < array[3] || array[3] < array[5])) {
            double n4 = (double)(array[1] - 2.0f * array[3] + array[5]);
            if (n4 != 0.0) {
                double n5 = (double)(array[1] - array[3]) / n4;
                if (n5 < 1.0 && n5 > 0.0) {
                    if (n > 0) {
                        if (array3[0] > n5) {
                            array3[n++] = array3[0];
                            array3[0] = n5;
                        }
                        else if (array3[0] < n5) {
                            array3[n++] = n5;
                        }
                    }
                    else {
                        array3[n++] = n5;
                    }
                }
            }
        }
        switch (n) {
            case 0: {
                break;
            }
            case 1: {
                ProcessFirstMonotonicPartOfQuad(processHandler, array, array2, (float)array3[0]);
                break;
            }
            case 2: {
                ProcessFirstMonotonicPartOfQuad(processHandler, array, array2, (float)array3[0]);
                double n6 = array3[1] - array3[0];
                if (n6 > 0.0) {
                    ProcessFirstMonotonicPartOfQuad(processHandler, array, array2, (float)(n6 / (1.0 - array3[0])));
                    break;
                }
                break;
            }
        }
        ProcessMonotonicQuad(processHandler, array, array2);
    }
    
    private static void ProcessFirstMonotonicPartOfQuad(ProcessPath.ProcessHandler processHandler, float[] array, int[] array2, float n) {
        float[] array3 = { array[0], array[1], array[0] + n * (array[2] - array[0]), array[1] + n * (array[3] - array[1]), 0.0f, 0.0f };
        array[2] += n * (array[4] - array[2]);
        array[3] += n * (array[5] - array[3]);
        array[0] = (array3[4] = array3[2] + n * (array[2] - array3[2]));
        array[1] = (array3[5] = array3[3] + n * (array[3] - array3[3]));
        ProcessMonotonicQuad(processHandler, array3, array2);
    }
    
    private static void DrawMonotonicCubic(ProcessPath.ProcessHandler processHandler, float[] array, boolean b, int[] array2) {
        int n = (int)(array[0] * 1024.0f);
        int n2 = (int)(array[1] * 1024.0f);
        int n3 = (int)(array[6] * 1024.0f);
        int n4 = (int)(array[7] * 1024.0f);
        int n5 = (n & 0x3FF) << 6;
        int n6 = (n2 & 0x3FF) << 6;
        int n7 = 32768;
        int n8 = 262144;
        int i = 8;
        int n9 = 6;
        int n10 = (int)((-array[0] + 3.0f * array[2] - 3.0f * array[4] + array[6]) * 128.0f);
        int n11 = (int)((-array[1] + 3.0f * array[3] - 3.0f * array[5] + array[7]) * 128.0f);
        int n12 = (int)((3.0f * array[0] - 6.0f * array[2] + 3.0f * array[4]) * 2048.0f);
        int n13 = (int)((3.0f * array[1] - 6.0f * array[3] + 3.0f * array[5]) * 2048.0f);
        int n14 = (int)((-3.0f * array[0] + 3.0f * array[2]) * 8192.0f);
        int n15 = (int)((-3.0f * array[1] + 3.0f * array[3]) * 8192.0f);
        int n16 = 6 * n10;
        int n17 = 6 * n11;
        int a = n16 + n12;
        int a2 = n17 + n13;
        int a3 = n10 + (n12 >> 1) + n14;
        int a4 = n11 + (n13 >> 1) + n15;
        int n18 = n & 0xFFFFFC00;
        int n19 = n2 & 0xFFFFFC00;
        int n20 = n3 - n;
        int n21 = n4 - n2;
        while (i > 0) {
            while (Math.abs(a) > n8 || Math.abs(a2) > n8) {
                a = (a << 1) - n16;
                a2 = (a2 << 1) - n17;
                a3 = (a3 << 2) - (a >> 1);
                a4 = (a4 << 2) - (a2 >> 1);
                i <<= 1;
                n8 <<= 3;
                n7 <<= 3;
                n5 <<= 3;
                n6 <<= 3;
                n9 += 3;
            }
            while ((i & 0x1) == 0x0 && n9 > 6 && Math.abs(a3) <= n7 && Math.abs(a4) <= n7) {
                a3 = (a3 >> 2) + (a >> 3);
                a4 = (a4 >> 2) + (a2 >> 3);
                a = a + n16 >> 1;
                a2 = a2 + n17 >> 1;
                i >>= 1;
                n8 >>= 3;
                n7 >>= 3;
                n5 >>= 3;
                n6 >>= 3;
                n9 -= 3;
            }
            if (--i > 0) {
                n5 += a3;
                n6 += a4;
                a3 += a;
                a4 += a2;
                a += n16;
                a2 += n17;
                n = n18 + (n5 >> n9);
                n2 = n19 + (n6 >> n9);
                if ((n3 - n ^ n20) < 0) {
                    n = n3;
                }
                if ((n4 - n2 ^ n21) < 0) {
                    n2 = n4;
                }
                processHandler.processFixedLine(n, n2, n, n2, array2, b, false);
            }
            else {
                processHandler.processFixedLine(n, n2, n3, n4, array2, b, false);
            }
        }
    }
    
    private static void ProcessMonotonicCubic(ProcessPath.ProcessHandler processHandler, float[] array, int[] array2) {
        float[] array3 = new float[8];
        float n2;
        float n = n2 = array[0];
        float n4;
        float n3 = n4 = array[1];
        for (int i = 2; i < 8; i += 2) {
            n2 = ((n2 > array[i]) ? array[i] : n2);
            n = ((n < array[i]) ? array[i] : n);
            n4 = ((n4 > array[i + 1]) ? array[i + 1] : n4);
            n3 = ((n3 < array[i + 1]) ? array[i + 1] : n3);
        }
        if (processHandler.clipMode == 0) {
            if (processHandler.dhnd.xMaxf < n2 || processHandler.dhnd.xMinf > n || processHandler.dhnd.yMaxf < n4 || processHandler.dhnd.yMinf > n3) {
                return;
            }
        }
        else {
            if (processHandler.dhnd.yMaxf < n4 || processHandler.dhnd.yMinf > n3 || processHandler.dhnd.xMaxf < n2) {
                return;
            }
            if (processHandler.dhnd.xMinf > n) {
                int n5 = 0;
                int n6 = 2;
                int n7 = 4;
                int n8 = 6;
                float xMinf = processHandler.dhnd.xMinf;
                array[n7] = (array[n8] = xMinf);
                array[n5] = (array[n6] = xMinf);
            }
        }
        if (n - n2 > 256.0f || n3 - n4 > 256.0f) {
            array3[6] = array[6];
            array3[7] = array[7];
            array3[4] = (array[4] + array[6]) / 2.0f;
            array3[5] = (array[5] + array[7]) / 2.0f;
            float n9 = (array[2] + array[4]) / 2.0f;
            float n10 = (array[3] + array[5]) / 2.0f;
            array3[2] = (n9 + array3[4]) / 2.0f;
            array3[3] = (n10 + array3[5]) / 2.0f;
            array[2] = (array[0] + array[2]) / 2.0f;
            array[3] = (array[1] + array[3]) / 2.0f;
            array[4] = (array[2] + n9) / 2.0f;
            array[5] = (array[3] + n10) / 2.0f;
            array[6] = (array3[0] = (array[4] + array3[2]) / 2.0f);
            array[7] = (array3[1] = (array[5] + array3[3]) / 2.0f);
            ProcessMonotonicCubic(processHandler, array, array2);
            ProcessMonotonicCubic(processHandler, array3, array2);
        }
        else {
            DrawMonotonicCubic(processHandler, array, processHandler.dhnd.xMinf > n2 || processHandler.dhnd.xMaxf < n || processHandler.dhnd.yMinf > n4 || processHandler.dhnd.yMaxf < n3, array2);
        }
    }
    
    private static void ProcessCubic(ProcessPath.ProcessHandler processHandler, float[] array, int[] array2) {
        double[] a = new double[4];
        double[] array3 = new double[3];
        double[] array4 = new double[2];
        int toIndex = 0;
        if ((array[0] > array[2] || array[2] > array[4] || array[4] > array[6]) && (array[0] < array[2] || array[2] < array[4] || array[4] < array[6])) {
            array3[2] = (double)(-array[0] + 3.0f * array[2] - 3.0f * array[4] + array[6]);
            array3[1] = (double)(2.0f * (array[0] - 2.0f * array[2] + array[4]));
            array3[0] = (double)(-array[0] + array[2]);
            for (int solveQuadratic = QuadCurve2D.solveQuadratic(array3, array4), i = 0; i < solveQuadratic; ++i) {
                if (array4[i] > 0.0 && array4[i] < 1.0) {
                    a[toIndex++] = array4[i];
                }
            }
        }
        if ((array[1] > array[3] || array[3] > array[5] || array[5] > array[7]) && (array[1] < array[3] || array[3] < array[5] || array[5] < array[7])) {
            array3[2] = (double)(-array[1] + 3.0f * array[3] - 3.0f * array[5] + array[7]);
            array3[1] = (double)(2.0f * (array[1] - 2.0f * array[3] + array[5]));
            array3[0] = (double)(-array[1] + array[3]);
            for (int solveQuadratic2 = QuadCurve2D.solveQuadratic(array3, array4), j = 0; j < solveQuadratic2; ++j) {
                if (array4[j] > 0.0 && array4[j] < 1.0) {
                    a[toIndex++] = array4[j];
                }
            }
        }
        if (toIndex > 0) {
            Arrays.sort(a, 0, toIndex);
            ProcessFirstMonotonicPartOfCubic(processHandler, array, array2, (float)a[0]);
            for (int k = 1; k < toIndex; ++k) {
                double n = a[k] - a[k - 1];
                if (n > 0.0) {
                    ProcessFirstMonotonicPartOfCubic(processHandler, array, array2, (float)(n / (1.0 - a[k - 1])));
                }
            }
        }
        ProcessMonotonicCubic(processHandler, array, array2);
    }
    
    private static void ProcessFirstMonotonicPartOfCubic(ProcessPath.ProcessHandler processHandler, float[] array, int[] array2, float n) {
        float[] array3 = new float[8];
        array3[0] = array[0];
        array3[1] = array[1];
        float n2 = array[2] + n * (array[4] - array[2]);
        float n3 = array[3] + n * (array[5] - array[3]);
        array3[2] = array[0] + n * (array[2] - array[0]);
        array3[3] = array[1] + n * (array[3] - array[1]);
        array3[4] = array3[2] + n * (n2 - array3[2]);
        array3[5] = array3[3] + n * (n3 - array3[3]);
        array[4] += n * (array[6] - array[4]);
        array[5] += n * (array[7] - array[5]);
        array[2] = n2 + n * (array[4] - n2);
        array[3] = n3 + n * (array[5] - n3);
        array[0] = (array3[6] = array3[4] + n * (array[2] - array3[4]));
        array[1] = (array3[7] = array3[5] + n * (array[3] - array3[5]));
        ProcessMonotonicCubic(processHandler, array3, array2);
    }
    
    private static void ProcessLine(ProcessPath.ProcessHandler processHandler, float n, float n2, float n3, float n4, int[] array) {
        float[] array2 = { n, n2, n3, n4, 0.0f, 0.0f };
        float xMinf = processHandler.dhnd.xMinf;
        float yMinf = processHandler.dhnd.yMinf;
        float xMaxf = processHandler.dhnd.xMaxf;
        float yMaxf = processHandler.dhnd.yMaxf;
        int testandclip = TESTANDCLIP(yMinf, yMaxf, array2, 1, 0, 3, 2);
        if (testandclip == 4) {
            return;
        }
        int is_CLIPPED = IS_CLIPPED(testandclip);
        int testandclip2 = TESTANDCLIP(yMinf, yMaxf, array2, 3, 2, 1, 0);
        if (testandclip2 == 4) {
            return;
        }
        int is_CLIPPED2 = IS_CLIPPED(testandclip2);
        int n5 = is_CLIPPED || is_CLIPPED2;
        if (processHandler.clipMode == 0) {
            int testandclip3 = TESTANDCLIP(xMinf, xMaxf, array2, 0, 1, 2, 3);
            if (testandclip3 == 4) {
                return;
            }
            int n11 = n5 || IS_CLIPPED(testandclip3);
            int testandclip4 = TESTANDCLIP(xMinf, xMaxf, array2, 2, 3, 0, 1);
            if (testandclip4 == 4) {
                return;
            }
            int n12 = is_CLIPPED2 || IS_CLIPPED(testandclip4);
            processHandler.processFixedLine((int)(array2[0] * 1024.0f), (int)(array2[1] * 1024.0f), (int)(array2[2] * 1024.0f), (int)(array2[3] * 1024.0f), array, n11 || n12, (boolean)n12);
        }
        else {
            int clipclamp = CLIPCLAMP(xMinf, xMaxf, array2, 0, 1, 2, 3, 4, 5);
            int n6 = (int)(array2[0] * 1024.0f);
            int n7 = (int)(array2[1] * 1024.0f);
            if (clipclamp == 0) {
                processHandler.processFixedLine((int)(array2[4] * 1024.0f), (int)(array2[5] * 1024.0f), n6, n7, array, false, (boolean)is_CLIPPED2);
            }
            else if (clipclamp == 4) {
                return;
            }
            int clipclamp2 = CLIPCLAMP(xMinf, xMaxf, array2, 2, 3, 0, 1, 4, 5);
            int n8 = is_CLIPPED2 || clipclamp2 == 1;
            int n9 = (int)(array2[2] * 1024.0f);
            int n10 = (int)(array2[3] * 1024.0f);
            processHandler.processFixedLine(n6, n7, n9, n10, array, false, (boolean)n8);
            if (clipclamp2 == 0) {
                processHandler.processFixedLine(n9, n10, (int)(array2[4] * 1024.0f), (int)(array2[5] * 1024.0f), array, false, (boolean)n8);
            }
        }
        return;
    }
    
    private static boolean doProcessPath(ProcessPath.ProcessHandler processHandler, Path2D.Float float1, float n, float n2) {
        float[] array = new float[8];
        float[] array2 = new float[8];
        float[] array3 = { 0.0f, 0.0f };
        float[] array4 = new float[2];
        int[] array5 = new int[5];
        boolean n3 = false;
        int n4 = 0;
        array5[0] = 0;
        processHandler.dhnd.adjustBounds(-1048576, -1048576, 1048576, 1048576);
        if (processHandler.dhnd.strokeControl == 2) {
            array3[1] = (array3[0] = -0.5f);
            n = (float)((double)n - 0.5);
            n2 = (float)((double)n2 - 0.5);
        }
        PathIterator pathIterator = float1.getPathIterator(null);
        while (!pathIterator.isDone()) {
            switch (pathIterator.currentSegment(array)) {
                case 0: {
                    if (n3 && n4 == 0) {
                        if (processHandler.clipMode == 1 && (array2[0] != array3[0] || array2[1] != array3[1])) {
                            ProcessLine(processHandler, array2[0], array2[1], array3[0], array3[1], array5);
                        }
                        processHandler.processEndSubPath();
                    }
                    array2[0] = array[0] + n;
                    array2[1] = array[1] + n2;
                    if (array2[0] < 8.5070587E37f && array2[0] > -8.5070587E37f && array2[1] < 8.5070587E37f && array2[1] > -8.5070587E37f) {
                        n3 = true;
                        n4 = 0;
                        array3[0] = array2[0];
                        array3[1] = array2[1];
                    }
                    else {
                        n4 = 1;
                    }
                    array5[0] = 0;
                    break;
                }
                case 1: {
                    int n5 = 2;
                    float n6 = array[0] + n;
                    array2[n5] = n6;
                    float n7 = n6;
                    int n8 = 3;
                    float n9 = array[1] + n2;
                    array2[n8] = n9;
                    float n10 = n9;
                    if (n7 < 8.5070587E37f && n7 > -8.5070587E37f && n10 < 8.5070587E37f && n10 > -8.5070587E37f) {
                        if (n4 != 0) {
                            array2[0] = (array3[0] = n7);
                            array2[1] = (array3[1] = n10);
                            n3 = true;
                            n4 = 0;
                            break;
                        }
                        ProcessLine(processHandler, array2[0], array2[1], array2[2], array2[3], array5);
                        array2[0] = n7;
                        array2[1] = n10;
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 2: {
                    array2[2] = array[0] + n;
                    array2[3] = array[1] + n2;
                    int n11 = 4;
                    float n12 = array[2] + n;
                    array2[n11] = n12;
                    float n13 = n12;
                    int n14 = 5;
                    float n15 = array[3] + n2;
                    array2[n14] = n15;
                    float n16 = n15;
                    if (n13 < 8.5070587E37f && n13 > -8.5070587E37f && n16 < 8.5070587E37f && n16 > -8.5070587E37f) {
                        if (n4 != 0) {
                            array2[0] = (array3[0] = n13);
                            array2[1] = (array3[1] = n16);
                            n3 = true;
                            n4 = 0;
                            break;
                        }
                        if (array2[2] < 8.5070587E37f && array2[2] > -8.5070587E37f && array2[3] < 8.5070587E37f && array2[3] > -8.5070587E37f) {
                            ProcessQuad(processHandler, array2, array5);
                        }
                        else {
                            ProcessLine(processHandler, array2[0], array2[1], array2[4], array2[5], array5);
                        }
                        array2[0] = n13;
                        array2[1] = n16;
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 3: {
                    array2[2] = array[0] + n;
                    array2[3] = array[1] + n2;
                    array2[4] = array[2] + n;
                    array2[5] = array[3] + n2;
                    int n17 = 6;
                    float n18 = array[4] + n;
                    array2[n17] = n18;
                    float n19 = n18;
                    int n20 = 7;
                    float n21 = array[5] + n2;
                    array2[n20] = n21;
                    float n22 = n21;
                    if (n19 < 8.5070587E37f && n19 > -8.5070587E37f && n22 < 8.5070587E37f && n22 > -8.5070587E37f) {
                        if (n4 != 0) {
                            array2[0] = (array3[0] = array2[6]);
                            array2[1] = (array3[1] = array2[7]);
                            n3 = true;
                            n4 = 0;
                            break;
                        }
                        if (array2[2] < 8.5070587E37f && array2[2] > -8.5070587E37f && array2[3] < 8.5070587E37f && array2[3] > -8.5070587E37f && array2[4] < 8.5070587E37f && array2[4] > -8.5070587E37f && array2[5] < 8.5070587E37f && array2[5] > -8.5070587E37f) {
                            ProcessCubic(processHandler, array2, array5);
                        }
                        else {
                            ProcessLine(processHandler, array2[0], array2[1], array2[6], array2[7], array5);
                        }
                        array2[0] = n19;
                        array2[1] = n22;
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 4: {
                    if (n3 && n4 == 0) {
                        n4 = 0;
                        if (array2[0] != array3[0] || array2[1] != array3[1]) {
                            ProcessLine(processHandler, array2[0], array2[1], array3[0], array3[1], array5);
                            array2[0] = array3[0];
                            array2[1] = array3[1];
                        }
                        processHandler.processEndSubPath();
                        break;
                    }
                    else {
                        break;
                    }
                }
            }
            pathIterator.next();
        }
        if ((n3 & ((n4 == 0) ? 1 : 0)) != 0x0) {
            if (processHandler.clipMode == 1 && (array2[0] != array3[0] || array2[1] != array3[1])) {
                ProcessLine(processHandler, array2[0], array2[1], array3[0], array3[1], array5);
            }
            processHandler.processEndSubPath();
        }
        return true;
    }
    
    private static void FillPolygon(ProcessPath.FillProcessHandler fillProcessHandler, int n) {
        int n2 = fillProcessHandler.dhnd.xMax - 1;
        ProcessPath.FillData fd = fillProcessHandler.fd;
        int plgYMin = fd.plgYMin;
        int plgYMax = fd.plgYMax;
        int n3 = (plgYMax - plgYMin >> 10) + 4;
        int n4 = plgYMin - 1 & 0xFFFFFC00;
        int n5 = (n == 1) ? -1 : 1;
        List<Object> plgPnts = fd.plgPnts;
        int size = plgPnts.size();
        if (size <= 1) {
            return;
        }
        ProcessPath.Point[] array = new ProcessPath.Point[n3];
        ((ProcessPath.Point)plgPnts.get(0)).prev = null;
        for (int i = 0; i < size - 1; ++i) {
            ProcessPath.Point prev = (ProcessPath.Point)plgPnts.get(i);
            ProcessPath.Point next = (ProcessPath.Point)plgPnts.get(i + 1);
            int n6 = prev.y - n4 - 1 >> 10;
            prev.nextByY = array[n6];
            array[n6] = prev;
            prev.next = next;
            next.prev = prev;
        }
        ProcessPath.Point point = (ProcessPath.Point)plgPnts.get(size - 1);
        int n7 = point.y - n4 - 1 >> 10;
        point.nextByY = array[n7];
        array[n7] = point;
        ProcessPath.ActiveEdgeList o = new ProcessPath.ActiveEdgeList();
        for (int n8 = n4 + 1024, n9 = 0; n8 <= plgYMax && n9 < n3; n8 += 1024, ++n9) {
            for (ProcessPath.Point nextByY = array[n9]; nextByY != null; nextByY = nextByY.nextByY) {
                if (nextByY.prev != null && !nextByY.prev.lastPoint) {
                    if (nextByY.prev.edge != null && nextByY.prev.y <= n8) {
                        o.delete(nextByY.prev.edge);
                        nextByY.prev.edge = null;
                    }
                    else if (nextByY.prev.y > n8) {
                        o.insert(nextByY.prev, n8);
                    }
                }
                if (!nextByY.lastPoint && nextByY.next != null) {
                    if (nextByY.edge != null && nextByY.next.y <= n8) {
                        o.delete(nextByY.edge);
                        nextByY.edge = null;
                    }
                    else if (nextByY.next.y > n8) {
                        o.insert(nextByY, n8);
                    }
                }
            }
            if (!o.isEmpty()) {
                o.sort();
                int n10 = 0;
                int n11 = 0;
                int xMin = fillProcessHandler.dhnd.xMin;
                for (ProcessPath.Edge edge = o.head; edge != null; edge.x += edge.dx, edge = edge.next) {
                    n10 += edge.dir;
                    if ((n10 & n5) != 0x0 && n11 == 0) {
                        xMin = edge.x + 1024 - 1 >> 10;
                        n11 = 1;
                    }
                    if ((n10 & n5) == 0x0 && n11 != 0) {
                        int n12 = edge.x - 1 >> 10;
                        if (xMin <= n12) {
                            fillProcessHandler.dhnd.drawScanline(xMin, n12, n8 >> 10);
                        }
                        n11 = 0;
                    }
                }
                if (n11 != 0 && xMin <= n2) {
                    fillProcessHandler.dhnd.drawScanline(xMin, n2, n8 >> 10);
                }
            }
        }
        return;
    }
    
    static int access$100(int n, int n2, int[] array, int n3, int n4, int n5, int n6) {
        return TESTANDCLIP(n, n2, array, n3, n4, n5, n6);
    }
    
    static boolean access$200(int n) {
        return IS_CLIPPED(n);
    }
    
    static int access$300(int n, int n2, int[] array, int n3, int n4, int n5, int n6, int n7, int n8) {
        return CLIPCLAMP(n, n2, array, n3, n4, n5, n6, n7, n8);
    }
    
    static {
        ProcessPath.noopEndSubPathHandler = new ProcessPath.EndSubPathHandler() {
            @Override
            public void processEndSubPath() {
            }
        };
    }
    
    public abstract static class DrawHandler
    {
        public int xMin;
        public int yMin;
        public int xMax;
        public int yMax;
        public float xMinf;
        public float yMinf;
        public float xMaxf;
        public float yMaxf;
        public int strokeControl;
        
        public DrawHandler(int n, int n2, int n3, int n4, int n5) {
            this.setBounds(n, n2, n3, n4, n5);
        }
        
        public void setBounds(int xMin, int yMin, int xMax, int yMax) {
            this.xMin = xMin;
            this.yMin = yMin;
            this.xMax = xMax;
            this.yMax = yMax;
            this.xMinf = (float)xMin - 0.5f;
            this.yMinf = (float)yMin - 0.5f;
            this.xMaxf = (float)xMax - 0.5f - 9.765625E-4f;
            this.yMaxf = (float)yMax - 0.5f - 9.765625E-4f;
        }
        
        public void setBounds(int n, int n2, int n3, int n4, int strokeControl) {
            this.strokeControl = strokeControl;
            this.setBounds(n, n2, n3, n4);
        }
        
        public void adjustBounds(int xMin, int yMin, int xMax, int yMax) {
            if (this.xMin > xMin) {
                xMin = this.xMin;
            }
            if (this.xMax < xMax) {
                xMax = this.xMax;
            }
            if (this.yMin > yMin) {
                yMin = this.yMin;
            }
            if (this.yMax < yMax) {
                yMax = this.yMax;
            }
            this.setBounds(xMin, yMin, xMax, yMax);
        }
        
        public DrawHandler(int n, int n2, int n3, int n4) {
            this(n, n2, n3, n4, 0);
        }
        
        public abstract void drawLine(int p0, int p1, int p2, int p3);
        
        public abstract void drawPixel(int p0, int p1);
        
        public abstract void drawScanline(int p0, int p1, int p2);
    }
    
    public abstract static class ProcessHandler implements ProcessPath.EndSubPathHandler
    {
        ProcessPath.DrawHandler dhnd;
        int clipMode;
        
        public ProcessHandler(ProcessPath.DrawHandler dhnd, int clipMode) {
            this.dhnd = dhnd;
            this.clipMode = clipMode;
        }
        
        public abstract void processFixedLine(int p0, int p1, int p2, int p3, int[] p4, boolean p5, boolean p6);
    }
    
    private static class DrawProcessHandler extends ProcessPath.ProcessHandler
    {
        ProcessPath.EndSubPathHandler processESP;
        
        public DrawProcessHandler(ProcessPath.DrawHandler dhnd, ProcessPath.EndSubPathHandler processESP) {
            super(dhnd, 0);
            this.dhnd = dhnd;
            this.processESP = processESP;
        }
        
        @Override
        public void processEndSubPath() {
            this.processESP.processEndSubPath();
        }
        
        void PROCESS_LINE(int n, int n2, int n3, int n4, boolean b, int[] array) {
            int n5 = n >> 10;
            int n6 = n2 >> 10;
            int n7 = n3 >> 10;
            int n8 = n4 >> 10;
            if (((n5 ^ n7) | (n6 ^ n8)) == 0x0) {
                if (b && (this.dhnd.yMin > n6 || this.dhnd.yMax <= n6 || this.dhnd.xMin > n5 || this.dhnd.xMax <= n5)) {
                    return;
                }
                if (array[0] == 0) {
                    array[array[0] = 1] = n5;
                    array[2] = n6;
                    array[3] = n5;
                    array[4] = n6;
                    this.dhnd.drawPixel(n5, n6);
                }
                else if ((n5 != array[3] || n6 != array[4]) && (n5 != array[1] || n6 != array[2])) {
                    this.dhnd.drawPixel(n5, n6);
                    array[3] = n5;
                    array[4] = n6;
                }
                return;
            }
            else {
                if ((!b || (this.dhnd.yMin <= n6 && this.dhnd.yMax > n6 && this.dhnd.xMin <= n5 && this.dhnd.xMax > n5)) && array[0] == 1 && ((array[1] == n5 && array[2] == n6) || (array[3] == n5 && array[4] == n6))) {
                    this.dhnd.drawPixel(n5, n6);
                }
                this.dhnd.drawLine(n5, n6, n7, n8);
                if (array[0] == 0) {
                    array[array[0] = 1] = n5;
                    array[2] = n6;
                    array[3] = n5;
                    array[4] = n6;
                }
                if ((array[1] == n7 && array[2] == n8) || (array[3] == n7 && array[4] == n8)) {
                    if (b && (this.dhnd.yMin > n8 || this.dhnd.yMax <= n8 || this.dhnd.xMin > n7 || this.dhnd.xMax <= n7)) {
                        return;
                    }
                    this.dhnd.drawPixel(n7, n8);
                }
                array[3] = n7;
                array[4] = n8;
                return;
            }
        }
        
        void PROCESS_POINT(int n, int n2, boolean b, int[] array) {
            int n3 = n >> 10;
            int n4 = n2 >> 10;
            if (b && (this.dhnd.yMin > n4 || this.dhnd.yMax <= n4 || this.dhnd.xMin > n3 || this.dhnd.xMax <= n3)) {
                return;
            }
            if (array[0] == 0) {
                array[array[0] = 1] = n3;
                array[2] = n4;
                array[3] = n3;
                array[4] = n4;
                this.dhnd.drawPixel(n3, n4);
            }
            else if ((n3 != array[3] || n4 != array[4]) && (n3 != array[1] || n4 != array[2])) {
                this.dhnd.drawPixel(n3, n4);
                array[3] = n3;
                array[4] = n4;
            }
            return;
        }
        
        @Override
        public void processFixedLine(int n, int n2, int n3, int n4, int[] array, boolean b, boolean b2) {
            int n5 = (n ^ n3) | (n2 ^ n4);
            if ((n5 & 0xFFFFFC00) == 0x0) {
                if (n5 == 0) {
                    this.PROCESS_POINT(n + 512, n2 + 512, b, array);
                }
                return;
            }
            else {
                int n13;
                int n14;
                int n16;
                int n17;
                if (n == n3 || n2 == n4) {
                    n13 = n + 512;
                    n16 = n3 + 512;
                    n14 = n2 + 512;
                    n17 = n4 + 512;
                }
                else {
                    int n6 = n3 - n;
                    int n7 = n4 - n2;
                    int n8 = n & 0xFFFFFC00;
                    int n9 = n2 & 0xFFFFFC00;
                    int n10 = n3 & 0xFFFFFC00;
                    int n11 = n4 & 0xFFFFFC00;
                    if (n8 == n || n9 == n2) {
                        n13 = n + 512;
                        n14 = n2 + 512;
                    }
                    else {
                        n8 = ((n < n3) ? (n8 + 1024) : n8);
                        n9 = ((n2 < n4) ? (n9 + 1024) : n9);
                        int n12 = n2 + (n8 - n) * n7 / n6;
                        if (n12 >= n9 && n12 <= n9 + 1024) {
                            n13 = n8;
                            n14 = n12 + 512;
                        }
                        else {
                            n13 = n + (n9 - n2) * n6 / n7 + 512;
                            n14 = n9;
                        }
                    }
                    if (n10 == n3 || n11 == n4) {
                        n16 = n3 + 512;
                        n17 = n4 + 512;
                    }
                    else {
                        n10 = ((n > n3) ? (n10 + 1024) : n10);
                        n11 = ((n2 > n4) ? (n11 + 1024) : n11);
                        int n15 = n4 + (n10 - n3) * n7 / n6;
                        if (n15 >= n11 && n15 <= n11 + 1024) {
                            n16 = n10;
                            n17 = n15 + 512;
                        }
                        else {
                            n16 = n3 + (n11 - n4) * n6 / n7 + 512;
                            n17 = n11;
                        }
                    }
                }
                this.PROCESS_LINE(n13, n14, n16, n17, b, array);
                return;
            }
        }
    }
    
    private static class Point
    {
        public int x;
        public int y;
        public boolean lastPoint;
        public ProcessPath.Point prev;
        public ProcessPath.Point next;
        public ProcessPath.Point nextByY;
        public ProcessPath.Edge edge;
        
        public Point(int x, int y, boolean lastPoint) {
            this.x = x;
            this.y = y;
            this.lastPoint = lastPoint;
        }
    }
    
    private static class Edge
    {
        int x;
        int dx;
        ProcessPath.Point p;
        int dir;
        ProcessPath.Edge prev;
        ProcessPath.Edge next;
        
        public Edge(ProcessPath.Point p4, int x, int dx, int dir) {
            this.p = p4;
            this.x = x;
            this.dx = dx;
            this.dir = dir;
        }
    }
    
    private static class FillData
    {
        List<ProcessPath.Point> plgPnts;
        public int plgYMin;
        public int plgYMax;
        
        public FillData() {
            this.plgPnts = new Vector(256);
        }
        
        public void addPoint(int n, int n2, boolean b) {
            if (this.plgPnts.size() == 0) {
                this.plgYMax = n2;
                this.plgYMin = n2;
            }
            else {
                this.plgYMin = ((this.plgYMin > n2) ? n2 : this.plgYMin);
                this.plgYMax = ((this.plgYMax < n2) ? n2 : this.plgYMax);
            }
            this.plgPnts.add(new ProcessPath.Point(n, n2, b));
        }
        
        public boolean isEmpty() {
            return this.plgPnts.size() == 0;
        }
        
        public boolean isEnded() {
            return ((ProcessPath.Point)this.plgPnts.get(this.plgPnts.size() - 1)).lastPoint;
        }
        
        public boolean setEnded() {
            return ((ProcessPath.Point)this.plgPnts.get(this.plgPnts.size() - 1)).lastPoint = true;
        }
    }
    
    private static class ActiveEdgeList
    {
        ProcessPath.Edge head;
        
        public boolean isEmpty() {
            return this.head == null;
        }
        
        public void insert(ProcessPath.Point point, int n) {
            ProcessPath.Point next = point.next;
            int x = point.x;
            int y = point.y;
            int x2 = next.x;
            int y2 = next.y;
            if (y == y2) {
                return;
            }
            int n2 = x2 - x;
            int n3 = y2 - y;
            int n4;
            int n5;
            if (y < y2) {
                n4 = n - y;
                n5 = -1;
            }
            else {
                x = x2;
                n4 = n - y2;
                n5 = 1;
            }
            int n6;
            int n7;
            if ((float)n2 > 1048576.0f || (float)n2 < -1048576.0f) {
                n6 = (int)((double)n2 * 1024.0 / (double)n3);
                n7 = x + (int)((double)n2 * (double)n4 / (double)n3);
            }
            else {
                n6 = (n2 << 10) / n3;
                n7 = x + n2 * n4 / n3;
            }
            ProcessPath.Edge head = new ProcessPath.Edge(point, n7, n6, n5);
            head.next = this.head;
            head.prev = null;
            if (this.head != null) {
                this.head.prev = head;
            }
            point.edge = head;
            this.head = head;
            return;
        }
        
        public void delete(ProcessPath.Edge edge) {
            ProcessPath.Edge prev = edge.prev;
            ProcessPath.Edge next = edge.next;
            if (prev != null) {
                prev.next = next;
            }
            else {
                this.head = next;
            }
            if (next != null) {
                next.prev = prev;
            }
        }
        
        public void sort() {
            ProcessPath.Edge edge = null;
            int n = 1;
            while (edge != this.head.next && n != 0) {
                ProcessPath.Edge head;
                ProcessPath.Edge next = head = this.head;
                ProcessPath.Edge edge2 = next.next;
                n = 0;
                while (next != edge) {
                    if (next.x >= edge2.x) {
                        n = 1;
                        if (next == this.head) {
                            ProcessPath.Edge next3 = edge2.next;
                            edge2.next = next;
                            next.next = next3;
                            this.head = edge2;
                            head = edge2;
                        }
                        else {
                            ProcessPath.Edge next2 = edge2.next;
                            edge2.next = next;
                            next.next = next2;
                            head.next = edge2;
                            head = edge2;
                        }
                    }
                    else {
                        head = next;
                        next = next.next;
                    }
                    edge2 = next.next;
                    if (edge2 != edge) {
                        continue;
                    }
                    edge = next;
                }
            }
            ProcessPath.Edge edge3 = this.head;
            ProcessPath.Edge prev = null;
            while (edge3 != null) {
                edge3.prev = prev;
                prev = edge3;
                edge3 = edge3.next;
            }
        }
    }
    
    private static class FillProcessHandler extends ProcessPath.ProcessHandler
    {
        ProcessPath.FillData fd;
        
        @Override
        public void processFixedLine(int n, int n2, int n3, int n4, int[] array, boolean b, boolean b2) {
            if (b) {
                int[] array2 = { n, n2, n3, n4, 0, 0 };
                int n5 = (int)(this.dhnd.xMinf * 1024.0f);
                int n6 = (int)(this.dhnd.xMaxf * 1024.0f);
                int n7 = (int)(this.dhnd.yMinf * 1024.0f);
                int n8 = (int)(this.dhnd.yMaxf * 1024.0f);
                if (ProcessPath.access$100(n7, n8, array2, 1, 0, 3, 2) == 4) {
                    return;
                }
                int access$100 = ProcessPath.access$100(n7, n8, array2, 3, 2, 1, 0);
                if (access$100 == 4) {
                    return;
                }
                int access$101 = ProcessPath.access$200(access$100);
                int access$102 = ProcessPath.access$300(n5, n6, array2, 0, 1, 2, 3, 4, 5);
                if (access$102 == 0) {
                    this.processFixedLine(array2[4], array2[5], array2[0], array2[1], array, false, (boolean)access$101);
                }
                else if (access$102 == 4) {
                    return;
                }
                int access$103 = ProcessPath.access$300(n5, n6, array2, 2, 3, 0, 1, 4, 5);
                int n9 = access$101 || access$103 == 1;
                this.processFixedLine(array2[0], array2[1], array2[2], array2[3], array, false, (boolean)n9);
                if (access$103 == 0) {
                    this.processFixedLine(array2[2], array2[3], array2[4], array2[5], array, false, (boolean)n9);
                }
                return;
            }
            else {
                if (this.fd.isEmpty() || this.fd.isEnded()) {
                    this.fd.addPoint(n, n2, false);
                }
                this.fd.addPoint(n3, n4, false);
                if (b2) {
                    this.fd.setEnded();
                }
                return;
            }
        }
        
        FillProcessHandler(ProcessPath.DrawHandler drawHandler) {
            super(drawHandler, 1);
            this.fd = new ProcessPath.FillData();
        }
        
        @Override
        public void processEndSubPath() {
            if (!this.fd.isEmpty()) {
                this.fd.setEnded();
            }
        }
    }
    
    public interface EndSubPathHandler
    {
        public abstract void processEndSubPath();
    }
}
